#!/usr/bin/env node

const program = require('commander');
const fs = require('fs')
const copy = require('kopy')
const path = require('path')
const Handlebars = require('handlebars')
const chalk = require('chalk')
const rimraf = require('rimraf')

let data = []
let dest = ''

program
  .version('1.0.0')
  .option('-c, --config [config file name]', 'configs')
  .option('-d, --dest [dest folder name]', 'dest folder', 'dest')
  .parse(process.argv);

if (program.config) {
  data = require(path.resolve(process.cwd(), program.config))
} else {
  console.log(chalk.red('error: must provide config file name'))
  process.exit(1)
}

if (program.dest) {
  dest = path.resolve(process.cwd(), program.dest)
} else {
  dest = path.resolve(process.cwd(), 'generated-api')
}

if (fs.existsSync(dest)) {
  rimraf(dest, () => fs.mkdirSync(dest))
} else {
  fs.mkdirSync(dest)
}

const handlerSource = fs.readFileSync(path.resolve(__dirname, './template/src/handlers.hbs'), 'utf8')
const mongoModelSource = fs.readFileSync(path.resolve(__dirname, './template/src/mongo-models.hbs'), 'utf8')
const postgresModelSource = fs.readFileSync(path.resolve(__dirname, './template/src/postgres-models.hbs'), 'utf8')
const postgresTablesSource = fs.readFileSync(path.resolve(__dirname, './template/src/postgres-tables.hbs'), 'utf8')
const routeSource = fs.readFileSync(path.resolve(__dirname, './template/src/routes.hbs'), 'utf8')
const schemaSource = fs.readFileSync(path.resolve(__dirname, './template/src/schemas.hbs'), 'utf8')

const handlerTemplate = Handlebars.compile(handlerSource)
const mongoModelTemplate = Handlebars.compile(mongoModelSource)
const postgresModeTlemplate = Handlebars.compile(postgresModelSource)
const routeTemplate = Handlebars.compile(routeSource)
const schemaTemplate = Handlebars.compile(schemaSource)
const postgresTablesTemplate = Handlebars.compile(postgresTablesSource)

const writeFile = (filePath, content)  => fs.writeFile(filePath, content, function(err) {
  if (err) {
    return console.log(err);
  }
  console.log(chalk.green(`${filePath} was saved!`))
})

copy(path.resolve(__dirname, './template'), dest, {
  clean: true,
  glob:[
    '**',
    '!**/node_modules/**',
    '!**/*.hbs/**',
  ]
}).then(() => {
  writeFile(`${dest}/src/postgres/tables.js`, postgresTablesTemplate(data))
  data.forEach(item => {
    writeFile(`${dest}/src/handlers/${item.fileName}.js`, handlerTemplate(item))
    writeFile(`${dest}/src/mongoose/models/${item.fileName}.js`, mongoModelTemplate(item))
    writeFile(`${dest}/src/postgres/models/${item.fileName}.js`, postgresModeTlemplate(item))
    writeFile(`${dest}/src/routes/${item.fileName}.js`, routeTemplate(item))
    writeFile(`${dest}/src/schemas/${item.fileName}.js`, schemaTemplate(item))
  })
}).catch(err => {
  console.log(err.stack)
})

