const fp = require('fastify-plugin')
const mongoose = require('mongoose')
const bluebird = require('bluebird')
const glob = require('glob')
const config = require('../configs/env')

let models = {}
mongoose.Promise = bluebird

mongoose.plugin(require('../mongoose/plugin'))

const fastifyMongoose = (fastify, options = {}, next) => {
  mongoose.connect(config.mongo.host, { useMongoClient: true })
  const db = mongoose.connection
  const { Schema } = mongoose

  db.on('error', err => {
    console.log(['error', 'mongoose'], err)
  })

  db.once('open', () => {
    console.log(['info', 'mongoose'], 'Mongoose connected.')
    glob('../mongoose/models/**/*.js', { cwd: __dirname }, (err, files) => {
      if (err) {
        throw err
      }
      for (let file of files) {
        const model = require(file)(mongoose, Schema)
        models[model.modelName] = model
      }
      fastify.decorateRequest('models', models)
      next()
    })
  })
}

module.exports = fp(fastifyMongoose)
