const R = require('ramda')
const buildInColumns = ['_id', 'createdAt', 'updatedAt', '__v']

module.exports = (schema, options) => {
  schema.statics = R.merge(schema.statics, {
    async get (id) {
      try {
        const model = await this.findById(id)
        if (model) {
          return model
        }
        const err = new Error()
        err.message = 'not such model exists'
        return Promise.reject(err)
      } catch (error) {
        throw error
      }
    },
    async findAndUpdate (id, model) {
      return this.findOneAndUpdate({
        _id: id,
      }, {
        $set: R.omit(['_id'], model),
      }, {
        new: true,
      })
    },
    async findAndRemove (id) {
      return this.findOneAndRemove({ _id: id })
    },
    async exists (condition) {
      const result = await this.find(condition)
      return result.length > 0
    },
    async list (query) {
      const pageIndex = Number.parseInt(query.pageIndex, 10) || 1
      const pageSize = Number.parseInt(query.pageSize, 10) || 10
      const customerColumns = R.difference(R.keys(this.schema.paths), buildInColumns)
      const search = R.pick(customerColumns)(query)
      const totalCount = await this.find(search).count()
      const data = await this.find(search)
        .sort({ createdAt: -1 })
        .skip((pageIndex - 1) * pageSize)
        .limit(pageSize)
      return { pageIndex, pageSize, totalCount, data }
    },
  })
}
