const R = require('ramda')

module.exports = (tableName, knex, fastify) => {
  return {
    create: async (body) => {
      const data = await knex(tableName).insert(body).returning('*')
      return data
    },
    get: async (id) => {
      const [data] = await knex(tableName).where('_id', id)
      return data || fastify.boom(404)
    },
    findAndUpdate: async (id, body) => {
      const [data] = await knex(tableName).where('_id', id).update({...body, updatedAt: new Date()}).returning('*')
      return data
    },
    findAndRemove: async (id) => {
      const result = await knex(tableName).where('_id', id).del()
      return fastify.boom(result === 1 ? 200 : 404)
    },
    findOne: async (query) => {
      const [result] = await knex(tableName).where(query)
      return result
    },
    list: async (query) => {
      const { pageSize = 10, pageIndex = 1 } = query
      const condition = R.omit(['pageSize', 'pageIndex'])(query)
      const [{ count: totalCount }] = await knex(tableName).where(condition).count('*')
      const data = await knex.select('*')
        .from(tableName)
        .where(condition)
        .limit(pageSize)
        .offset((pageIndex - 1) * pageSize)
        .orderBy('_id', 'desc')
      return { pageSize, pageIndex, totalCount, data }
    },
  }
}
