const request = require('request')
const { endpoint } = require('./config')
const R = require('ramda')

let model = {
  _id: 0,
  memberName: '1',
  password: '1',
  avatar: '1',
  phoneNumber: '1',
}

const postMember = () => new Promise((resolve, reject) => {
  request.post(`${endpoint}/members`, {
    json: R.omit(['_id'], model),
  }, function (err, httpResponse, body) {
    err ? reject(err) : resolve(body)
  })
})

const putMember = () => new Promise((resolve, reject) => {
  request.put(`${endpoint}/members/${model._id}`, {
    json: R.omit(['_id'], model),
  }, function (err, httpResponse, body) {
    err ? reject(err) : resolve(body)
  })
})

const deleteMember = () => new Promise((resolve, reject) => {
  request.delete(`${endpoint}/members/${model._id}`, {
    json: {},
  }, function (err, httpResponse, body) {
    err ? reject(err) : resolve(body)
  })
})

const listMember = () => new Promise((resolve, reject) => {
  request.get(`${endpoint}/members?pageIndex=1&pageSize=10`, {
    json: {},
  }, function (err, httpResponse, body) {
    err ? reject(err) : resolve(body)
  })
})

test('#POST# member', async () => {
  const data = await postMember()
  expect(data._id).toBeGreaterThan(0)
  model._id = data._id
})

test('#PUT# member', async () => {
  const data = await putMember()
  expect(data.statusCode).toBe(200)
})

test('#DELETE# member', async () => {
  const data = await deleteMember()
  expect(data.statusCode).toBe(200)
})

test('#LIST# member', async () => {
  const data = await listMember()
  expect(data).toBeDefined()
})
